package de.bsvrz.sys.funclib.bitctrl.modell.util.cache;

import com.bitctrl.util.monitor.IMonitor;

import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;

/**
 * Cached zu einer PID das entsprechende Modell-Objekt.
 * 
 * @author BitCtrl Systems GmbH, schnepel
 * @version $Id$
 */
public class ModellObjektCache extends AbstractCache {

	private Object threadSync;

	private SystemObjekt modellObjekt;

	private AbstractDavVerbindungsCache verbindungsCache;

	public ModellObjektCache(final String pid,
			final AbstractDavVerbindungsCache verbindungsCache) {
		super(pid, verbindungsCache);
	}

	protected void preInit(final AbstractDavVerbindungsCache verbindungsCache) {
		threadSync = new Object();
		this.verbindungsCache = verbindungsCache;
	}

	@Override
	protected boolean doInit(final IMonitor monitor) {
		modellObjekt = verbindungsCache.getObjektFactory().getModellobjekt(
				getName());
		return null != modellObjekt;
	}

	@Override
	protected void callDoInit(final IMonitor monitor) {
		synchronized (threadSync) {
			final Thread thread = new Thread() {
				@Override
				public void run() {
					synchronized (threadSync) {
						ModellObjektCache.super.callDoInit(monitor);
					}
				}
			};
			thread.setDaemon(true);
			thread.start();
		}
	}

	public SystemObjekt getModellObjekt() {
		ensureInit();
		if (null == modellObjekt) {
			throw new IllegalStateException("Der ModellObjektCache("
					+ getName() + ") wurde nicht korrekt initialisiert.");
		}
		return modellObjekt;
	}
}
