/*
 * Allgemeine Funktionen für die Verwendung von Objekten aus dem Datenkatalog-Modell.
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.sys.funclib.bitctrl.modell.util.rdstmc;

import java.util.List;

/**
 * Listener, welcher über Änderungen der Landesmeldestellen bekannten
 * RDS-Meldungen informiert.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: RdsMeldungenChangedListener.java 22700 2010-04-23 12:00:20Z
 *          meissner $
 */
public interface RdsMeldungenChangedListener {

	/**
	 * Informiert über neu hinzugefügte RDS-Meldungen.
	 * 
	 * @param rdsMeldungen
	 *            die neu hinzugefügten RDS-Meldungen
	 */
	void rdsMeldungenAdded(List<RdsMeldungWrapper> rdsMeldungen);

	/**
	 * Informiert über entfernte RDS-Meldungen.
	 * 
	 * @param rdsMeldungen
	 *            die entfernten RDS-Meldungen
	 */
	void rdsMeldungenRemoved(List<RdsMeldungWrapper> rdsMeldungen);

	/**
	 * Informiert über eine RDS-Meldung, deren Daten sich geändert haben.
	 * 
	 * @param neuerWert
	 *            der neue EngstellenWrapper
	 * @param alterWert
	 *            der alte EngstellenWrapper
	 */
	void rdsMeldungChanged(RdsMeldungWrapper neuerWert,
			RdsMeldungWrapper alterWert);
}
